/**
 * Point.js - implement a point object, with variations
 * @author  Tom Legrady
 * @date    Dec. 24,  2011
 */
function Point( x, y ) {
    "use strict";

    this.x = x;
    this.y = y;
}
Point.method( 'toString', function() {
    "use strict";
    return '(' + this.x + ', ' + this.y + ')';
} );

Point.method( 'distance', function( p ) {
    var dx = this.x - p.x,
    dy = this.y - p.y;

    return Math.sqrt( dx * dx +  dy * dy );
} );

function BoundedPoint( x, y ) {
    "use strict";
    var bx = Math.min( Math.max( LEGRADY.bounds.minX, x),
                         LEGRADY.bounds.maxX );
    var by = Math.min( Math.max( LEGRADY.bounds.minY, y),
                         LEGRADY.bounds.maxY );
    Point.call( this, bx, by );
}

BoundedPoint.inheritsFrom( Point );
BoundedPoint.method( 'toString', function() {
    "use strict";
    return '[' + this.x + ', ' + this.y + ']';
});
BoundedPoint.method( 'oldtoString', function() {
    "use strict";
    return this.uber();
});

function RandomPoint( f ) {
    "use strict";
    
    if ( f === undefined ) {
        f = Math.random;
    }
    var x = Math.round( f() * LEGRADY.bounds.maxX);
    var y = Math.round( f() * LEGRADY.bounds.maxY);
    Point.call( this, x, y );
}
RandomPoint.inheritsFrom( Point );
// ----------------------------------------------------------------------
// End of File.
//
