#!/usr/bin/perl

use warnings;
use strict;

my $pi = atan2( 0, -1);
my $eps = 1 / 1_000;
my $inf = 1_000_000;

# my ( $l1, $l2 ) = ( { p1 => { x => 0, y => 3 },
# 		      p2 => { x => -1.5, y => 0 }
# 		  }, {
# 		      p1 => { x => 0, y => 7 },
# 		      p2 => { x => 14, y => 0 },
# 		  },
# 		    );

my ( $l1, $l2 ) = ( { p1 => { x => 10, y => 31 },
		      p2 => { x => 40, y => 41 }
		  }, {
		      p1 => { x => 25, y => 25 },
		      p2 => { x => 32, y => 20 },
		  },
		    );

$l1->{theta} = atan2( ( $l1->{p2}{y} - $l1->{p1}{y} ), ( $l1->{p2}{x} - $l1->{p1}{x} ));
$l2->{theta} = atan2( ( $l2->{p2}{y} - $l2->{p1}{y} ), ( $l2->{p2}{x} - $l2->{p1}{x} ));

$l1->{thperp} = $l1->{theta} + ($l1->{theta} > 0 ? ($pi / -2) : ($pi / 2) );
$l2->{thperp} = $l2->{theta} + ($l2->{theta} > 0 ? ($pi / -2) : ($pi / 2) );


printf "theta: %7.5f %7.5f\n",  $l1->{theta} ,  $l2->{theta} ;
printf "PI/? : %7.5f %7.5f\n",  $pi / $l1->{theta} , $pi / $l2->{theta} ;
printf "perp: %7.5f %7.5f\n",  $l1->{thperp} ,  $l2->{thperp} ;

$l1->{slope} = abs( $l1->{p2}{x} - $l1->{p1}{x} ) < $eps ? $inf : 
    ( $l1->{p2}{y} - $l1->{p1}{y} ) / ( $l1->{p2}{x} - $l1->{p1}{x} );
$l2->{slope} = abs( $l2->{p2}{x} - $l2->{p1}{x} ) < $eps ? $inf : 
    ( $l2->{p2}{y} - $l2->{p1}{y} ) / ( $l2->{p2}{x} - $l2->{p1}{x} );

printf "slope: %7.5f %7.5f\n",  $l1->{slope} ,  $l2->{slope} ;

$l1->{invslope} = abs($l1->{slope}) < $eps ? $inf : -1 / $l1->{slope};
$l2->{invslope} = abs($l2->{slope}) < $eps ? $inf : -1 / $l2->{slope};
printf "invsl: %7.5f %7.5f\n",  $l1->{invslope} ,  $l2->{invslope} ;

$l1->{b} = $l1->{p1}{y} - $l1->{p1}{x} * $l1->{slope};
$l2->{b} = $l2->{p1}{y} - $l2->{p1}{x} * $l2->{slope};
printf "b    : %7.5f %7.5f\n",  $l1->{b} ,  $l2->{b} ;

my $denom = $l1->{slope} - $l2->{slope};
print "denom is $denom\n";

my $p = ( $l2->{b} - $l1->{b} ) / $denom;
my $q = $l1->{slope} * $p + $l1->{b};

print "p is $p\nq is $q\n";

printf "point (p,q) %s on line 1\n",
    pointIsOnLine( $l1, {x => $p, y => $q } ) ? 'is' : 'is not';
printf "point (p,q) %s on line 2\n",
    pointIsOnLine( $l2, {x => $p, y => $q } ) ? 'is' : 'is not';

sub pointIsOnLine {
    my ( $l, $p ) = @_;

    return
	unless ( $p->{x} >= $l->{p1}{x}
		 && $p->{x} <= $l->{p2}{x});

    return
	unless ( $p->{y} >= $l->{p1}{y}
		 && $p->{y} <= $l->{p2}{y});

    my $dx = $p->{x} - $l->{p1}{x};
    return 1
	if $dx * $l->{slope} + $l->{p1}{y} - $p->{y} < $eps;
}
